#ifndef PANEL_GAMESS_H
#define PANEL_GAMESS_H

#ifndef PANEL_H
#include "Panel.h"
#endif

#ifndef DYNAMIC_SINGLETON_H
#include "Dynamic_singleton.h"
#endif

#ifndef UI_PANEL_GAMESS_H
#include "ui_Panel_GAMESS.h"
#endif

#include <QProcess>
#include <QFile>

namespace MM
{
class Model;
class Main_window;

class Panel_GAMESS 
:
    public Panel,
    public Ui_Panel_GAMESS,
    public Dynamic_singleton <Panel_GAMESS>
{
    Q_OBJECT

    Model *         model_;
    QProcess *      process_;
    Text            gamess_dir_;
    QFile           output_;
    Main_window *   player_;

    bool            read_atom_coords_flag_;
    bool            read_charges_flag_;
    bool            read_moments_flag_;
    bool            user_stop_flag_;
    bool            success_;
    Text            external_basis_; 
    int             optimization_it_;

public:    
    Panel_GAMESS ();
   ~Panel_GAMESS ();

public slots:
    void            show_options (const QString & text);

    void            basis_panel ();

    void            run   ();
    void            check ();
    void            tune  ();
    void            start ();
    void            stop  ();

    void            started ();
    void            finished (int exitCode, QProcess::ExitStatus exitStatus);

    void            read ();

protected slots:
    void            set_BSSE_calc (bool);
    void            extrapolate ();

protected:
    Text const &    gamess_dir () const {return gamess_dir_;}
    Text const &    setup_gamess_dir ();

    void            read_atom_coords (char *buf);
    void            read_charges     (char *buf);
    void            read_moments     (char *buf);
    void            create_input     (char const* first_string);

    Text            basis ();
    Text            data  ();

    bool            not_semiempirical () const;
	bool            is_spherical_basis_set () const;
    void            set_stop_state ();
    void            set_run_state  ();

    void            turn (bool on);
    void            closeEvent (QCloseEvent * event);
};

}//MM

#endif //PANEL_GAMESS_H
